@@ -37,8 +37,8 @@ class BuildConfigurationInformation(object):
     """
     version_major = 1
     version_minor = 8
-    version_patch = 9
-    version_so_patch = 2
+    version_patch = 13
+    version_so_patch = 13
     version_suffix = ''
 
     version_string = '%d.%d.%d%s' % (
@@ -104,6 +104,11 @@ def process_command_line(args):
         formatter = IndentedHelpFormatter(max_help_position = 50),
         version = BuildConfigurationInformation.version_string)
 
+    parser.add_option('--verbose', action='store_true', default=False,
+                      help='Show debug messages')
+    parser.add_option('--quiet', action='store_true', default=False,
+                      help='Show only warnings and errors')
+
     target_group = OptionGroup(parser, 'Target options')
 
     target_group.add_option('--cc', dest='compiler',
@@ -139,14 +144,14 @@ def process_command_line(args):
 
     build_group.add_option('--with-tr1-implementation', metavar='WHICH',
                            dest='with_tr1', default=None,
-                           help='enable TR1 (options: none, system, boost)')
+                           help='enable TR1 (choices: none, system, boost)')
 
     build_group.add_option('--with-build-dir',
                            metavar='DIR', default='',
                            help='setup the build in DIR')
 
     build_group.add_option('--makefile-style', metavar='STYLE', default=None,
-                           help='choose a makefile style (unix, nmake)')
+                           help='choose a makefile style (unix or nmake)')
 
     build_group.add_option('--with-local-config',
                            dest='local_config', metavar='FILE',
@@ -156,6 +161,10 @@ def process_command_line(args):
                            action='store_true', default=False,
                            help=SUPPRESS_HELP)
 
+    build_group.add_option('--link-method',
+                           default=None,
+                           help=SUPPRESS_HELP)
+
     mods_group = OptionGroup(parser, 'Module selection')
 
     mods_group.add_option('--enable-modules', dest='enabled_modules',
@@ -167,23 +176,27 @@ def process_command_line(args):
     mods_group.add_option('--no-autoload', action='store_true', default=False,
                           help='disable automatic loading')
 
-    for mod in ['openssl', 'gnump', 'bzip2', 'zlib']:
+    for lib in ['OpenSSL', 'GNU MP', 'Bzip2', 'Zlib']:
 
         # This is just an implementation of Optik's append_const action,
         # but that is not available in Python 2.4's optparse, so use a
         # callback instead
 
+        mod = lib.lower().replace(' ', '')
+
         def optparse_callback(option, opt, value, parser, dest, mod):
             parser.values.__dict__[dest].append(mod)
 
-        mods_group.add_option('--with-%s' % (mod),
+        mods_group.add_option('--with-%s' % mod,
+                              help='add support for using %s' % (lib),
                               action='callback',
                               callback=optparse_callback,
                               callback_kwargs = {
                                   'dest': 'enabled_modules', 'mod': mod }
                               )
 
-        mods_group.add_option('--without-%s' % (mod), help=SUPPRESS_HELP,
+        mods_group.add_option('--without-%s' % mod,
+                              help=SUPPRESS_HELP,
                               action='callback',
                               callback=optparse_callback,
                               callback_kwargs = {
@@ -469,10 +482,15 @@ class CompilerInfo(object):
 
         del self.mach_opt
 
-    def mach_abi_link_flags(self, osname, arch, submodel):
+    def mach_abi_link_flags(self, osname, arch, submodel, debug_p):
+
+        def all():
+            if debug_p:
+                return 'all-debug'
+            return 'all'
 
         abi_link = set()
-        for what in ['all', osname, arch, submodel]:
+        for what in [all(), osname, arch, submodel]:
             if self.mach_abi_linking.get(what) != None:
                 abi_link.add(self.mach_abi_linking.get(what))
 
@@ -495,6 +513,18 @@ class CompilerInfo(object):
 
         return ''
 
+    def library_opt_flags(self, options):
+        def gen_flags():
+            yield self.lib_opt_flags
+
+            if options.debug_build:
+                yield self.debug_flags
+
+            if not options.debug_build:
+                yield self.no_debug_flags
+
+        return (' '.join(gen_flags())).strip()
+
     def so_link_command_for(self, osname):
         if osname in self.so_link_flags:
             return self.so_link_flags[osname]
@@ -705,11 +735,10 @@ def create_template_vars(build_config, options, modules, cc, arch, osinfo):
 
         'mp_bits': choose_mp_bits(),
 
-        'cc': cc.binary_name + cc.mach_abi_link_flags(options.os,
-                                                      options.arch,
-                                                      options.cpu),
+        'cc': cc.binary_name + cc.mach_abi_link_flags(
+            options.os, options.arch, options.cpu, options.debug_build),
 
-        'lib_opt': cc.lib_opt_flags,
+        'lib_opt': cc.library_opt_flags(options),
         'mach_opt': cc.mach_opts(options.arch, options.cpu),
         'check_opt': cc.check_opt_flags,
         'lang_flags': cc.lang_flags + options.extra_flags,
@@ -903,6 +932,16 @@ def load_info_files(options):
     ccinfo = dict([(form_name(info), CompilerInfo(info))
                     for info in list_files_in_build_data('cc')])
 
+    def info_file_load_report(type, num):
+        if num > 0:
+            logging.debug('Loaded %d %s info files' % (num, type))
+        else:
+            logging.warning('Failed to load any %s info files' % (type))
+
+    info_file_load_report('CPU', len(archinfo));
+    info_file_load_report('OS', len(osinfo))
+    info_file_load_report('compiler', len(ccinfo))
+
     if 'defaults' in osinfo:
         del osinfo['defaults'] # FIXME (remove the file)
 
@@ -913,16 +952,35 @@ Perform the filesystem operations needed to setup the build
 """
 def setup_build(build_config, options, template_vars):
 
+    """
+    Choose the link method based on system availablity and user request
+    """
+    def choose_link_method(req_method):
+
+        def useable_methods():
+            if 'symlink' in os.__dict__:
+                yield 'symlink'
+            if 'link' in os.__dict__:
+                yield 'hardlink'
+            yield 'copy'
+
+        for method in useable_methods():
+            if req_method is None or req_method == method:
+                return method
+
+        logging.info('Could not use requested link method %s' % (req_method))
+        return 'copy'
+
     """
     Copy or link the file, depending on what the platform offers
     """
-    def portable_symlink(filename, target_dir):
+    def portable_symlink(filename, target_dir, method):
 
         if not os.access(filename, os.R_OK):
             logging.warning('Missing file %s' % (filename))
             return
 
-        if 'symlink' in os.__dict__:
+        if method == 'symlink':
             def count_dirs(dir, accum = 0):
                 if dir == '' or dir == os.path.curdir:
                     return accum
@@ -938,13 +996,16 @@ def setup_build(build_config, options, template_vars):
 
             os.symlink(source, target)
 
-        elif 'link' in os.__dict__:
+        elif method == 'hardlink':
             os.link(filename,
                     os.path.join(target_dir, os.path.basename(filename)))
 
-        else:
+        elif method == 'copy':
             shutil.copy(filename, target_dir)
 
+        else:
+            raise Exception('Unknown link method %s' % (method))
+
     def choose_makefile_template(style):
         if style == 'nmake':
             return 'nmake.in'
@@ -1004,8 +1065,13 @@ def setup_build(build_config, options, template_vars):
     logging.debug('Linking %d header files in %s' % (
         len(build_config.headers), build_config.full_include_dir))
 
+    link_method = choose_link_method(options.link_method)
+    logging.info('Using %s to link files into build directory' % (link_method))
+
     for header_file in build_config.headers:
-        portable_symlink(header_file, build_config.full_include_dir)
+        portable_symlink(header_file,
+                         build_config.full_include_dir,
+                         link_method)
 
 """
 Main driver
@@ -1014,9 +1080,18 @@ def main(argv = None):
     if argv is None:
         argv = sys.argv
 
+    options = process_command_line(argv[1:])
+
+    def log_level():
+        if options.verbose:
+            return logging.DEBUG
+        if options.quiet:
+            return logging.WARNING
+        return logging.INFO
+
     logging.basicConfig(stream = sys.stdout,
                         format = '%(levelname) 7s: %(message)s',
-                        level = logging.INFO)
+                        level = log_level())
 
     logging.debug('%s invoked with options "%s"' % (
         argv[0], ' '.join(argv[1:])))
@@ -1024,11 +1099,13 @@ def main(argv = None):
     logging.debug('Platform: OS="%s" machine="%s" proc="%s"' % (
         platform.system(), platform.machine(), platform.processor()))
 
-    options = process_command_line(argv[1:])
-
     if options.os == "java":
         raise Exception("Jython detected: need --os and --cpu to set target")
 
+    if re.match('^cygwin_.*', options.os):
+        logging.debug("Converting '%s' to 'cygwin'", options.os)
+        options.os = 'cygwin'
+
     options.base_dir = os.path.dirname(argv[0])
     options.src_dir = os.path.join(options.base_dir, 'src')
 
